package scenariodiagrameditor.features;

import org.eclipse.emf.common.util.EList;
import org.eclipse.graphiti.features.IAddFeature;
import org.eclipse.graphiti.features.IDirectEditingInfo;
import org.eclipse.graphiti.features.IFeatureProvider;
import org.eclipse.graphiti.features.context.IAddContext;
import org.eclipse.graphiti.features.impl.AbstractAddFeature;
import org.eclipse.graphiti.mm.algorithms.Polyline;
import org.eclipse.graphiti.mm.algorithms.Rectangle;
import org.eclipse.graphiti.mm.algorithms.RoundedRectangle;
import org.eclipse.graphiti.mm.algorithms.Text;
import org.eclipse.graphiti.mm.algorithms.styles.Orientation;
import org.eclipse.graphiti.mm.pictograms.ContainerShape;
import org.eclipse.graphiti.mm.pictograms.Diagram;
import org.eclipse.graphiti.mm.pictograms.PictogramElement;
import org.eclipse.graphiti.mm.pictograms.Shape;
import org.eclipse.graphiti.services.Graphiti;
import org.eclipse.graphiti.services.IGaService;
import org.eclipse.graphiti.services.IPeCreateService;
import org.eclipse.graphiti.util.IColorConstant;

import de.uks.se.scenariodiagram.ScenarioAttribute;
import de.uks.se.scenariodiagram.ScenarioObject;

public class AddScenarioObjectFeature extends AbstractAddFeature implements IAddFeature
{
	private static final IColorConstant OBJECT_FOREGROUND =
			IColorConstant.BLACK;

	private static final IColorConstant OBJECT_BACKGROUND =
			IColorConstant.WHITE;

	private static final int DEFAULT_WIDTH = 100;
	private static final int DEFAULT_HEIGHT = 50;

	public AddScenarioObjectFeature(IFeatureProvider fp) {
		super(fp);
	}

	@Override
	public boolean canAdd(IAddContext context) {
		return context.getNewObject() instanceof ScenarioObject && context.getTargetContainer() instanceof Diagram;
	}

	@Override
	public PictogramElement add(IAddContext context) {
		Diagram targetDiagram = (Diagram) context.getTargetContainer();
		IPeCreateService peCreateService = Graphiti.getPeCreateService();
		IGaService gaService = Graphiti.getGaService();
		
		ScenarioObject scenarioObject = (ScenarioObject) context.getNewObject();

		ContainerShape containerShape = peCreateService.createContainerShape(targetDiagram, true);
		Rectangle rectangle = gaService.createRectangle(containerShape);
		gaService.setLocationAndSize(rectangle, context.getX(), context.getY(), DEFAULT_WIDTH, DEFAULT_HEIGHT);
		rectangle.setFilled(false);
		rectangle.setLineWidth(3);
		
		//Line
		{
			Shape shape = peCreateService.createShape(containerShape, false);
			// create and set graphics algorithm
			Polyline polyline = gaService.createPolyline(shape, new int[] { 0, 20, DEFAULT_WIDTH, 20});
			polyline.setForeground(manageColor(OBJECT_FOREGROUND));
			polyline.setLineWidth(3);
			
			link(shape, scenarioObject);
		}

		//Text
		{
			Shape shape = peCreateService.createShape(containerShape, false);

			//object text
			String value = "";
			EList<ScenarioAttribute> attributes = scenarioObject.getAttributes();
			for(ScenarioAttribute tmpAttribute : attributes){        	
				if ("text".equals(tmpAttribute.getTag()))
				{
					value = tmpAttribute.getValue();
					break;
				}
			}
			
			Text text = gaService.createText(shape, value);
			text.setHorizontalAlignment(Orientation.ALIGNMENT_CENTER);
			text.setVerticalAlignment(Orientation.ALIGNMENT_TOP);
			gaService.setLocationAndSize(text, 0, 0, DEFAULT_WIDTH, 20);

			peCreateService.createChopboxAnchor(containerShape);
			
			link(shape, scenarioObject);
		}
		
		peCreateService.createChopboxAnchor(containerShape);
		
		link(containerShape, scenarioObject);

		return containerShape;
	}
}
