package scenariodiagrameditor.features;

import org.eclipse.graphiti.features.ICreateFeature;
import org.eclipse.graphiti.features.IFeatureProvider;
import org.eclipse.graphiti.features.context.ICreateContext;
import org.eclipse.graphiti.features.impl.AbstractCreateFeature;

import de.uks.se.scenariodiagram.ScenarioAttribute;
import de.uks.se.scenariodiagram.ScenarioLink;
import de.uks.se.scenariodiagram.ScenarioObject;
import de.uks.se.scenariodiagram.ScenariodiagramFactory;

public class CreateScenarioAttributeFeature extends AbstractCreateFeature implements ICreateFeature {

	private final String DEFAULT_TAG = "tag";
	private final String DEFAULT_VALUE = "value";
	
	public CreateScenarioAttributeFeature(IFeatureProvider fp) {
		super(fp, "ScenarioAttribute", "Creates a new ScenarioAttribute");
	}

	@Override
	public boolean canCreate(ICreateContext context) {
		if(getBusinessObjectForPictogramElement(context.getTargetContainer()) instanceof ScenarioObject){
			return true;
		}
		if(getBusinessObjectForPictogramElement(context.getTargetConnection()) instanceof ScenarioLink){
			return true;
		}
		return false;
	}

	@Override
	public Object[] create(ICreateContext context) {
		ScenarioAttribute newScenarioAttribute = ScenariodiagramFactory.eINSTANCE.createScenarioAttribute();
		newScenarioAttribute.setTag(DEFAULT_TAG);
		newScenarioAttribute.setValue(DEFAULT_VALUE);
		
		//scenario object
		if(getBusinessObjectForPictogramElement(context.getTargetContainer()) != null)
		{
			ScenarioObject object = (ScenarioObject)getBusinessObjectForPictogramElement(context.getTargetContainer());
			object.getAttributes().add(newScenarioAttribute);
		}
		//scenario link
		else
		{
			ScenarioLink link = (ScenarioLink)getBusinessObjectForPictogramElement(context.getTargetConnection());
			link.getAttributes().add(newScenarioAttribute);
		}
		
		addGraphicalRepresentation(context, newScenarioAttribute);
		
		return new Object[] { newScenarioAttribute };
	}
}
