package scenariodiagrameditor.features;

import org.eclipse.graphiti.features.ICreateConnectionFeature;
import org.eclipse.graphiti.features.IFeatureProvider;
import org.eclipse.graphiti.features.context.ICreateConnectionContext;
import org.eclipse.graphiti.features.context.impl.AddConnectionContext;
import org.eclipse.graphiti.features.impl.AbstractCreateConnectionFeature;
import org.eclipse.graphiti.mm.pictograms.Connection;
import org.eclipse.graphiti.mm.pictograms.PictogramElement;

import de.uks.se.scenariodiagram.ScenarioAttribute;
import de.uks.se.scenariodiagram.ScenarioLink;
import de.uks.se.scenariodiagram.ScenarioObject;
import de.uks.se.scenariodiagram.ScenariodiagramFactory;

public class CreateScenarioLinkConnectionFeature extends AbstractCreateConnectionFeature implements ICreateConnectionFeature {

	private final String DEFAULT_TAG = "name";
	private final String DEFAULT_VALUE = "has";
	
	public CreateScenarioLinkConnectionFeature(IFeatureProvider fp) {
		super(fp, "ScenarioLink", "Creates a new ScenarioLink between two ScenarioObjects");
	}

	@Override
	public boolean canStartConnection(ICreateConnectionContext context)
	{
		return getBusinessObjectForPictogramElement(context.getSourcePictogramElement()) instanceof ScenarioObject;
	}

	@Override
	public boolean canCreate(ICreateConnectionContext context)
	{
		PictogramElement sourcePictogramElement = context.getSourcePictogramElement();
		PictogramElement targetPictogramElement = context.getTargetPictogramElement();

		if (getBusinessObjectForPictogramElement(sourcePictogramElement) instanceof ScenarioObject && getBusinessObjectForPictogramElement(targetPictogramElement) instanceof ScenarioObject)
		{
			return true;
		}
		
		return false;
	}

	@Override
	public Connection create(ICreateConnectionContext context)
	{
		Connection newConnection = null;

		ScenarioLink newScenarioLink = ScenariodiagramFactory.eINSTANCE.createScenarioLink();
		ScenarioObject source = (ScenarioObject)getBusinessObjectForPictogramElement(context.getSourcePictogramElement());
		ScenarioObject target = (ScenarioObject)getBusinessObjectForPictogramElement(context.getTargetPictogramElement());
		newScenarioLink.setSource(source);
		newScenarioLink.setTarget(target);
		
		ScenarioAttribute nameAttribute = ScenariodiagramFactory.eINSTANCE.createScenarioAttribute();
		nameAttribute.setTag(DEFAULT_TAG);
		nameAttribute.setValue(DEFAULT_VALUE);
		newScenarioLink.getAttributes().add(nameAttribute);
		
		AddConnectionContext addContext = new AddConnectionContext(context.getSourceAnchor(), context.getTargetAnchor());
		addContext.setNewObject(newScenarioLink);
		newConnection = (Connection) getFeatureProvider().addIfPossible(addContext);
		
		getDiagram().eResource().getContents().add(newScenarioLink);

		return newConnection;
	}
}
