package scenariodiagrameditor.features;

import org.eclipse.graphiti.features.IFeatureProvider;
import org.eclipse.graphiti.features.context.IDirectEditingContext;
import org.eclipse.graphiti.features.impl.AbstractDirectEditingFeature;
import org.eclipse.graphiti.mm.algorithms.GraphicsAlgorithm;
import org.eclipse.graphiti.mm.algorithms.Text;
import org.eclipse.graphiti.mm.pictograms.PictogramElement;
import org.eclipse.graphiti.mm.pictograms.Shape;

import de.uks.se.scenariodiagram.ScenarioAttribute;

public class DirectEditScenarioAttributeFeature extends AbstractDirectEditingFeature{

	public DirectEditScenarioAttributeFeature(IFeatureProvider fp) {
		super(fp);
	}

	@Override
	public int getEditingType() {
		return TYPE_TEXT;
	}

	@Override
	public String getInitialValue(IDirectEditingContext context) {
		PictogramElement pe = context.getPictogramElement();
        ScenarioAttribute scenarioAttribute = (ScenarioAttribute) getBusinessObjectForPictogramElement(pe);
        String initialValue = scenarioAttribute.getTag() + ":" + scenarioAttribute.getValue();
        
        return initialValue;
	}
	
	@Override
	public boolean canDirectEdit(IDirectEditingContext context) {
        PictogramElement pe = context.getPictogramElement();
        Object object = getBusinessObjectForPictogramElement(pe);
        GraphicsAlgorithm ga = context.getGraphicsAlgorithm();

        if (object instanceof ScenarioAttribute && ga instanceof Text) {
            return true;
        }

        return false;
	}

	@Override
	public void setValue(String value, IDirectEditingContext context) {
        PictogramElement pe = context.getPictogramElement();
        ScenarioAttribute scenarioAttribute = (ScenarioAttribute) getBusinessObjectForPictogramElement(pe);
        
        if(value.contains(":"))
        {
        	String[] tagValue = value.split(":");
        	
        	scenarioAttribute.setTag(tagValue[0]);
        	scenarioAttribute.setValue(tagValue[1]);
        	
        	
        	Text text = (Text)context.getGraphicsAlgorithm();
            text.setValue(value);
     
            updatePictogramElement(((Shape) pe).getContainer());
        }   
	}

}
