package scenariodiagrameditor.features;

import org.eclipse.emf.common.util.EList;
import org.eclipse.graphiti.features.IFeatureProvider;
import org.eclipse.graphiti.features.context.IDirectEditingContext;
import org.eclipse.graphiti.features.impl.AbstractDirectEditingFeature;
import org.eclipse.graphiti.mm.algorithms.GraphicsAlgorithm;
import org.eclipse.graphiti.mm.algorithms.Text;
import org.eclipse.graphiti.mm.pictograms.PictogramElement;
import org.eclipse.graphiti.mm.pictograms.Shape;

import de.uks.se.scenariodiagram.ScenarioAttribute;
import de.uks.se.scenariodiagram.ScenarioDiagram;
import de.uks.se.scenariodiagram.ScenarioObject;

public class DirectEditScenarioObjectFeature extends AbstractDirectEditingFeature{

	public DirectEditScenarioObjectFeature(IFeatureProvider fp) {
		super(fp);
	}

	@Override
	public int getEditingType() {
		return TYPE_TEXT;
	}

	@Override
	public String getInitialValue(IDirectEditingContext context) {
        PictogramElement pe = context.getPictogramElement();
        ScenarioObject scenarioObject = (ScenarioObject) getBusinessObjectForPictogramElement(pe);
        String value = "";
        
        EList<ScenarioAttribute> attributes = scenarioObject.getAttributes();
        for(ScenarioAttribute tmpAttribute : attributes){        	
        	if ("text".equals(tmpAttribute.getTag()))
        	{
        		value = tmpAttribute.getValue();
        		break;
        	}
        }
        
        return value;
	}

	@Override
	public boolean canDirectEdit(IDirectEditingContext context) {
        PictogramElement pe = context.getPictogramElement();
        Object object = getBusinessObjectForPictogramElement(pe);
        GraphicsAlgorithm ga = context.getGraphicsAlgorithm();

        if (object instanceof ScenarioObject && ga instanceof Text) {
            return true;
        }

        return false;
	}

	@Override
	public void setValue(String value, IDirectEditingContext context) {
        PictogramElement pe = context.getPictogramElement();
        ScenarioObject scenarioObject = (ScenarioObject) getBusinessObjectForPictogramElement(pe);
        
        EList<ScenarioAttribute> attributes = scenarioObject.getAttributes();
        for(ScenarioAttribute tmpAttribute : attributes){        	
        	if ("text".equals(tmpAttribute.getTag()))
        	{
        		tmpAttribute.setValue(value);
        		break;
        	}
        }
 
        Text text = (Text)context.getGraphicsAlgorithm();
        text.setValue(value);
 
        updatePictogramElement(((Shape) pe).getContainer());
	}
}
